!ClarionSourceClass implementation code - updated for clarion 6
   MEMBER
   MAP
DebugClass    PROCEDURE
   END
   INCLUDE('ABSOURCE.INC')

SourceObject   &ClarionSourceClass

ClarionSourceClass.Init PROCEDURE(*STRING SourceLine)
  CODE
  SELF.OriginalLine &= SourceLine
  SELF.WordQ        &= NEW(WordQueue)
  SELF.DelimiterQ   &= NEW(DelimiterQueue)
  SourceObject      &=SELF
  IF SELF.Debug THEN START(DebugClass).

ClarionSourceClass.Kill PROCEDURE
  CODE
  DISPOSE(SELF.WordQ)
  DISPOSE(SELF.DelimiterQ)

ClarionSourceClass.LoadQueues PROCEDURE
Char             STRING(1)
WordInProgress   BYTE
EndOfLine        BYTE
Pos              USHORT
  CODE
  FREE(SELF.WordQ)
  FREE(SELF.DelimiterQ)
  CLEAR(SELF.WordQ)
  CLEAR(SELF.DelimiterQ)
  EndOfLine=False
  LOOP Pos = 1 TO SELF.LineLength          !read left to right
    Char = SELF.Line[Pos]
    CASE Char
    OF ''''                                !Skip quoted strings
      Pos=INSTRING('''',SELF.Line,1,Pos+1)
      IF NOT Pos THEN BREAK.               !If odd number of quotes, bail out
    OF '"'                                 !Skip quoted strings
      Pos=INSTRING('"',SELF.Line,1,Pos+1)
      IF NOT Pos THEN BREAK.               !If odd number of quotes, bail out
    OF '!'                                 !Skip comments
      BREAK
    OF '|'
      SELF.Continued=True
      SELF.LineLength=Pos-1
      BREAK
    END
    IF INSTRING(Char,SELF.Delimiters,1,1)  !If delimiter
      IF WordInProgress
        DO EndWord
      ELSE
        SELF.DelimiterQ.Delimiter=SELF.DelimiterQ.Delimiter & Char
      END
    ELSE                                   !If non-delimiter
      IF Pos=1 THEN WordInProgress=True.
      IF WordInProgress
        SELF.WordQ.Word=SELF.WordQ.Word & Char
      ELSE
        DO EndDelimiter
      END
    END
  END
  EndOfLine=True
  IF WordInProgress
    DO EndWord
  ELSE
    DO EndDelimiter
  END

EndWord ROUTINE
  WordInProgress=False
  IF EndOfLine
    SELF.WordQ.RightDelimiter=0
  ELSE
    SELF.WordQ.RightDelimiter=RECORDS(SELF.DelimiterQ)+1
  END
  ADD(SELF.WordQ)
  ASSERT(~ERRORCODE())
  CLEAR(SELF.WordQ)
  SELF.DelimiterQ.LeftWord=RECORDS(SELF.WordQ)
  IF ~EndOfLine THEN SELF.DelimiterQ.Delimiter = Char.

EndDelimiter ROUTINE
  WordInProgress=True
  IF EndOfLine
    SELF.DelimiterQ.RightWord=0
  ELSE
    SELF.DelimiterQ.RightWord=RECORDS(SELF.WordQ)+1
  END
  ADD(SELF.DelimiterQ)
  ASSERT(~ERRORCODE())
  CLEAR(SELF.DelimiterQ)
  SELF.WordQ.LeftDelimiter=RECORDS(SELF.DelimiterQ)
  IF ~EndOfLine THEN SELF.WordQ.Word = Char.
  
ClarionSourceClass.TakeLine PROCEDURE(<*STRING SourceLine>)
KeyWord STRING(10)
  CODE
  IF SourceLine
     IF SELF.Continued
       SELF.Line=SELF.Line[1:SELF.LineLength]&CLIP(LEFT(SourceLine))
     ELSE
       SELF.Line=CLIP(SourceLine)
     END
  ELSE
     IF SELF.Continued
       SELF.Line=SELF.Line[1:SELF.LineLength]&CLIP(LEFT(SELF.OriginalLine))
     ELSE
       SELF.Line=CLIP(SELF.OriginalLine)
     END
  END

  SELF.Reset
  SELF.LineLength  = LEN(SELF.Line)
  IF ~SELF.LineLength
    SELF.SourceType = Source:IsBlank
    RETURN SELF.SourceType
  END
  IF LEFT(SELF.Line,1) = '!'
    SELF.SourceType = Source:IsComment
    RETURN SELF.SourceType
  END
  SELF.SetLabel
  SELF.LoadQueues
  IF SELF.Continued
    SELF.SourceType = Source:IsContinued
    RETURN SELF.SourceType
  END
  KeyWord = UPPER(LEFT(SELF.Line[SELF.EndLabel : SELF.LineLength],LEN(KeyWord)))

  IF KeyWord[1:9] = 'PROCEDURE' OR KeyWord[1:8] = 'FUNCTION'
    SELF.SetParameters
    SELF.SourceType = Source:DeclaresProcedure
    RETURN SELF.SourceType
  END
  IF KeyWord[1:6] = 'CLASS,' OR KeyWord[1:6] = 'CLASS(' OR KeyWord[1:6] = 'CLASS '
    SELF.SetParentClass
    SELF.SetModule
    SELF.SourceType = Source:DeclaresClass
    RETURN SELF.SourceType
  END
  IF KeyWord[1:6] = 'QUEUE,' OR KeyWord[1:6] = 'QUEUE ' OR KeyWord[1:6] = 'QUEUE('
    SELF.SourceType = Source:DeclaresQueue
    RETURN SELF.SourceType
  END
!  IF KeyWord[1:6] = 'GROUP ' OR KeyWord[1:6] = 'GROUP,' OR KeyWord[1:6] = 'GROUP('
!    SELF.SourceType = Source:DeclaresGroup
!    RETURN SELF.SourceType
!  END
!  IF KeyWord[1:8] = 'INCLUDE('
!    SELF.SourceType = Source:IncludesFile
!    RETURN SELF.SourceType
!  END
!  IF KeyWord[1:5] = 'FILE,'
!    SELF.SourceType = Source:DeclaresFile
!    RETURN SELF.SourceType
!  END
  RETURN SELF.SourceType

ClarionSourceClass.Reset PROCEDURE
  CODE
  SELF.LineLength  = 0
  SELF.Continued   = False
  SELF.EndLabel    = 1
  SELF.StartScan   = 0
  SELF.Label       = ''
  SELF.ModuleName  = ''
  SELF.Parameters  = ''
  SELF.ParentClass = ''
  SELF.SourceType  = Source:Other

ClarionSourceClass.SetLabel PROCEDURE
  CODE
  IF SELF.Line[1]
    SELF.EndLabel   = INSTRING(' ',SELF.Line,1,1)
    SELF.Label = SELF.Line[1:SELF.EndLabel]
  END

ClarionSourceClass.SetModule PROCEDURE
IsModule  USHORT
  CODE
  IsModule = INSTRING('MODULE(''',UPPER(SELF.Line),1,1)
  IF IsModule
    IsModule+=8
    SELF.ModuleName = SELF.Line[IsModule : INSTRING(''')',SELF.Line,1,IsModule)-1]
  END

ClarionSourceClass.SetParameters PROCEDURE
Parameters  USHORT
  CODE
  Parameters = INSTRING('ROCEDURE(',UPPER(SELF.Line),1,1)
  IF ~Parameters
    Parameters = INSTRING('FUNCTION(',UPPER(SELF.Line),1,1)
  END
  IF Parameters
    Parameters+=8
    SELF.Parameters = SELF.Line[Parameters : INSTRING(')',SELF.Line,1,Parameters)]
  END

ClarionSourceClass.SetParentClass PROCEDURE
IsParent    USHORT
  CODE
  IsParent = INSTRING('CLASS(',UPPER(SELF.Line),1,1)
  IF IsParent
    IsParent+=6
    SELF.ParentClass = SELF.Line[IsParent : INSTRING(')',SELF.Line,1,IsParent)-1]
  END

ClarionSourceClass.NextWord PROCEDURE
  CODE
  SELF.StartScan+=1
  GET(SELF.WordQ,SELF.StartScan)
  IF ERRORCODE()
    RETURN ''
  ELSE
   RETURN SELF.WordQ.Word
  END

ClarionSourceClass.NextDataLabelPair     PROCEDURE(*STRING DataType, *STRING Label)
Word    STRING(WordLength)
  CODE
  IF ~SELF.Parameters THEN RETURN False.
  IF ~SELF.StartScan
    LOOP
      Word = UPPER(SELF.NextWord())
      IF Word = 'PROCEDURE' OR Word = 'FUNCTION' OR Word = '' THEN BREAK.
    END
  END
  LOOP 
    GET(SELF.DelimiterQ,SELF.StartScan)
    IF ERRORCODE()
      BREAK
    ELSE
      SELF.StartScan+=1
      IF ~SELF.DelimiterQ.Delimiter
        DataType = SELF.GetLeftWord()
        Label    = SELF.GetRightWord()
        IF DataType AND Label
          RETURN True
        END
      END
    END
  END
  RETURN False

ClarionSourceClass.NextWordDotWordPair     PROCEDURE(*STRING Word1, *STRING Word2)
  CODE
  SELF.StartScan+=1
  LOOP 
    GET(SELF.DelimiterQ,SELF.StartScan)
    IF ERRORCODE()
      BREAK
    ELSE
      IF SELF.DelimiterQ.Delimiter = '.'
        Word1 = SELF.GetLeftWord()
        Word2 = SELF.GetRightWord()
        IF Word1 AND Word2
          RETURN True
        ELSE 
          SELF.StartScan+=1
        END
      ELSE
        SELF.StartScan+=1
      END
    END
  END
  RETURN False

ClarionSourceClass.NextObjectReference PROCEDURE(*STRING Word1, *STRING Word2)
!returns the next SELF/PARENT[.property1.property2...].method/property construction
W1        STRING(WordLength)
W2        STRING(WordLength)
  CODE
  CLEAR(Word1)
  CLEAR(Word2)
  LOOP WHILE SELF.NextWordDotWordPair(W1,W2)
    IF Word1
      Word1=CLIP(Word1)&'.'&W1
    ELSE
      Word1=W1
    END
    Word2=W2
    IF SELF.GetRightDelimiter() <> '.' THEN BREAK.
  END
  IF Word1 AND Word2
    RETURN True
  ELSE
    RETURN False
  END

ClarionSourceClass.GetLeftDelimiter     PROCEDURE
  CODE
  GET(SELF.DelimiterQ,SELF.WordQ.LeftDelimiter)
  IF ERRORCODE()
    RETURN ''
  ELSE
    RETURN SELF.DelimiterQ.Delimiter
  END

ClarionSourceClass.GetRightDelimiter     PROCEDURE
  CODE
  GET(SELF.DelimiterQ,SELF.WordQ.RightDelimiter)
  IF ERRORCODE()
    RETURN ''
  ELSE
    RETURN SELF.DelimiterQ.Delimiter
  END

ClarionSourceClass.GetLeftWord     PROCEDURE
  CODE
  GET(SELF.WordQ,SELF.DelimiterQ.LeftWord)
  IF ERRORCODE()
    RETURN ''
  ELSE
    RETURN SELF.WordQ.Word
  END

ClarionSourceClass.GetRightWord     PROCEDURE
  CODE
  GET(SELF.WordQ,SELF.DelimiterQ.RightWord)
  IF ERRORCODE()
    RETURN ''
  ELSE
    RETURN SELF.WordQ.Word
  END

DebugClass PROCEDURE
DbugWindow WINDOW('Debugging...'),AT(,,217,211),FONT('MS Sans Serif',8,,FONT:regular),SYSTEM,GRAY,DOUBLE
       LIST,AT(6,115,125,91),USE(?WordList),HVSCROLL,FORMAT('[15L|M~Prior~92L|M~Word~S(100)20L|M~Next~]'), |
           FROM(' ')
       LIST,AT(141,116,69,90),USE(?DelimiterList),HVSCROLL,FORMAT('[15L|M~Prior~34C|M~Delimiter~S(100)20L|M~Next~]'), |
           FROM(' ')
       BUTTON('TakeLine'),AT(5,2,,12),USE(?TakeLine)
       BUTTON('Next Word'),AT(4,17,207,12),USE(?NextWord)
       BUTTON('Data_Label'),AT(4,31,207,12),USE(?Data_Label)
       BUTTON('Word.Word'),AT(4,45,207,12),USE(?WordWord)
       BUTTON('LeftWord'),AT(4,58,207,12),USE(?LeftWord)
       BUTTON('RightWord'),AT(4,72,207,12),USE(?RightWord)
       BUTTON('NextOOP'),AT(4,86,207,12),USE(?NextOOP)
       BUTTON('Close'),AT(179,2,,12),USE(?Close)
     END
  CODE
  OPEN(DbugWindow)
  ?WordList{PROP:From}=SourceObject.WordQ
  ?DelimiterList{PROP:From}=SourceObject.DelimiterQ
  ACCEPT
    CASE ACCEPTED()
    OF ?TakeLine
      SourceObject.TakeLine()
      IF SourceObject.Line
        DbugWindow{Prop:Text}=SourceObject.Line
      ELSE
        DbugWindow{Prop:Text}='blank line'
      END
    OF ?Data_Label
      IF SourceObject.NextDataLabelPair(data",label")
        ?Data_Label{PROP:Text}='DL: '&data"&label"
      ELSE
        ?Data_Label{PROP:Text}='DL: none'
      END
    OF ?WordWord
      IF SourceObject.NextWordDotWordPair(w1",w2")
        ?WordWord{PROP:Text}='w.w: '&w1"&w2"
      ELSE
        ?WordWord{PROP:Text}='w.w: none'
      END
    OF ?NextOOP
      IF SourceObject.NextObjectReference(w1",w2")
        ?NextOOP{PROP:Text}='oop: '&w1"&w2"
      ELSE
        ?NextOOP{PROP:Text}='oop: none'
      END
    OF ?NextWord ;?NextWord{PROP:Text}='Next: '&SourceObject.NextWord()
    OF ?LeftWord ;?LeftWord{PROP:Text}='Left: '&SourceObject.GetLeftWord()
    OF ?RightWord;?RightWord{PROP:Text}='Right: '&SourceObject.GetRightWord()
    OF ?Close    ;POST(EVENT:CloseWindow)
    END
  END


OMIT('EndTestProcedure')
TestParser      PROCEDURE
FilesOpened     BYTE
window          WINDOW('Debugging'),AT(,,101,22),FONT('MS Sans Serif',8,,FONT:regular),SYSTEM,GRAY,RESIZE,MDI
                  BUTTON('Next Line'),AT(7,4,,14),USE(?NextLine)
                  BUTTON('Close'),AT(59,4),USE(?Close)
                END
TestProcedure      CLASS(WindowManager)
Init              PROCEDURE(),BYTE,PROC,VIRTUAL
Kill              PROCEDURE(),BYTE,PROC,VIRTUAL
TakeAccepted      PROCEDURE(),BYTE,PROC,VIRTUAL
                END
Resizer         WindowResizeClass
  CODE
  GlobalResponse = TestProcedure.Run()

TestProcedure.Init PROCEDURE()
ReturnValue     BYTE,AUTO
  CODE
  PathName='absource.clw'
  SELF.Request = GlobalRequest
  ReturnValue = PARENT.Init()
  IF ReturnValue THEN RETURN ReturnValue.
  SELF.FirstField = ?NextLine
  SELF.VCRRequest &= VCRRequest
  SELF.Errors &= GlobalErrors
  CLEAR(GlobalRequest)
  CLEAR(GlobalResponse)
  SELF.AddItem(?Close,RequestCancelled)
  Relate:SourceIn.Open
  FilesOpened = True
  OPEN(window)
  SELF.Opened=True
  Source.Debug=True
  Source.Init(SRCIN:SourceLine)
  SET(SourceIn)
  Resizer.Init(AppStrategy:Spread,Resize:SetMinSize)
  SELF.AddItem(Resizer)
  Resizer.AutoTransparent=True
  Resizer.SetParentDefaults
  INIMgr.Fetch('TestParser',window)
  Resizer.Resize
  Resizer.Reset
  SELF.SetAlerts()
  RETURN ReturnValue

TestProcedure.Kill PROCEDURE()
ReturnValue          BYTE,AUTO
  CODE
  ReturnValue = PARENT.Kill()
  IF ReturnValue THEN RETURN ReturnValue.
  IF FilesOpened
    Relate:SourceIn.Close
  END
  IF SELF.Opened
    INIMgr.Update('TestParser',window)
  END
  Source.Kill
  RETURN ReturnValue

TestProcedure.TakeAccepted PROCEDURE()
ReturnValue          BYTE,AUTO
Looped BYTE
  CODE
  LOOP
    IF Looped
      RETURN Level:Notify
    ELSE
      Looped = 1
    END
    CASE ACCEPTED()
    OF ?NextLine
      Access:SourceIn.Next()
      0{Prop:Text}=SRCIN:SourceLine
      Source.TakeLine()
    END
  ReturnValue = PARENT.TakeAccepted()
    RETURN ReturnValue
  END
  ReturnValue = Level:Fatal
  RETURN ReturnValue

!EndTestProcedure
